//+------------------------------------------------------------------+
//|                                                     Profiler.mq4 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, TheXpert"
#property link      "theforexpert@gmail.com"

extern double    Lots         = 0;//0.1;
extern double    RiskPercentage=0;

extern int       SL           = 0;//100;
extern int       TP           = 0;//500;

extern int       Magic        = 333333;

extern int       TimesToRepeat= 3;
extern int       Slippage     = 3;

extern int       LifeTimeMinutes = 270;

#include <DragTrade_v_1.0/DragTrade_Trading.mqh>

extern int     Range = 1000;
extern double  Threshold = 5;
extern double  MinTarget = 200;

extern double  Blind = 50;

extern int     MaxCount = 3;

datetime LastTime;

double _Lows[];
double _Highs[];

int init()
{
   LastTime = 0;
   
   return(0);
}

int start()
{
   if (LastTime == Time[0]) return (0);
   LastTime = Time[0];
   
   ArrayResize(_Lows, 0);
   ArrayResize(_Highs, 0);
   
   double hi = High[iHighest(Symbol(), 0, MODE_HIGH, Range + 1)];
   double lo = Low[iLowest(Symbol(), 0, MODE_LOW, Range + 1)];
   
   double ProfileDn[100];
   double ProfileUp[100];
   double Profile[100];

   ArrayInitialize(ProfileDn, 0);
   ArrayInitialize(ProfileUp, 0);
   
   for (int i = 2; i < Range; i++)
   {
      if (High[i] > High[i + 1])
      {
         if (High[i] > High[i - 1]) 
         {
            int pos = (High[i] - lo)*99/(hi-lo);
            ProfileUp[pos]++;
         }
         else if (High[i + 1] == High[i])
         {
            pos = i + 2;
            while (High[pos] == High[i] && pos < Bars - 1)
            {
               pos++;
            }
            if (pos < Bars && High[pos] != 0)
            {
               if (High[pos] < High[i])
               {
                  pos = (High[i] - lo)*99/(hi-lo);
                  if (pos <= Range) ProfileUp[pos]++;
               }
            }
         }
      }
   }
   
   for (i = 2; i < Range; i++)
   {
      if (Low[i] < Low[i + 1])
      {
         if (Low[i] < Low[i - 1]) 
         {
            pos = (Low[i] - lo)*99/(hi-lo);
            ProfileDn[pos]++;
         }
         else if (Low[i + 1] == Low[i])
         {
            pos = i + 2;
            while (Low[pos] == Low[i] && pos < Bars - 1)
            {
               pos++;
            }
            if (pos < Bars && Low[pos] != 0)
            {
               if (Low[pos] > Low[i])
               {
                  pos = (Low[i] - lo)*99/(hi-lo);
                  if (pos <= Range) ProfileDn[pos]++;
               }
            }
         }
      }
   }
   
   int size = ArraySize(ProfileDn);
   for (i = 0; i < size; i++)
   {
      Profile[i] = ProfileUp[i] + ProfileDn[i];
   }
   
   
   for (i = 0; i < 5; i++)
   {
      Smooth(ProfileDn);
      Smooth(ProfileUp);
      Smooth(Profile);
   }
   
   double max = Profile[ArrayMaximum(Profile)];
   
   int start = 0;
   int end = 0;
   bool started = false;
   
   for (i = 0; i < size; i++)
   {
      if (!started)
      {
         if (Profile[i] < max/Threshold)
         {
            started = true;
            start = i;
            end = i;
         }
      }
      else
      {
         if (Profile[i] < max/Threshold)
         {
            end = i;
         }
         else
         {
            double target = (end - start)*(hi - lo)/99.0;
            if (target > MinTarget*Point)
            {
               AddTarget(lo + start*(hi - lo)/99.0, lo + end*(hi - lo)/99.0);
            }
            started = false;
         }
      }
   }
   
   Trade();
   
   DrawLevels("Agg", lo, hi, Profile, Gray);
   DrawLevels("Dn", lo, hi, ProfileDn, Red);
   DrawLevels("Up", lo, hi, ProfileUp, Green);
}

void Trade()
{
   RefreshRates();
   double dn;
   bool isDn = FindDown(dn, Bid);
   
   double up;
   bool isUp = FindUp(up, Ask);
   
   double upPrice = -1;
   double upTicket = -1;
   double dnPrice = -1;
   double dnTicket = -1;
   
   int upCount = 0;
   int dnCount = 0;
   
   int total = OrdersTotal();
   for(int i = 0; i < total; i++)
   {
      if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
      if (OrderMagicNumber() != Magic) continue;
      if (OrderSymbol() != Symbol()) continue;
      
      switch (OrderType())
      {
         case OP_BUY:
               upCount++;
            break;
         case OP_BUYSTOP:
               if (!isUp)
               {
                  Print("d1");
                  OrderDelete(OrderTicket());
               }
               
               upPrice = OrderOpenPrice();
               upTicket = OrderTicket();
               upCount++;
            break;
         case OP_SELL:
               dnCount++;
            break;
         case OP_SELLSTOP:
               if (!isDn) 
               {
                  Print("d2");
                  OrderDelete(OrderTicket());
               }

               dnPrice = OrderOpenPrice();
               dnTicket = OrderTicket();
               dnCount++;
            break;
      }
   }
   
   if (isUp)
   {
      if ((MathAbs(up - upPrice) > Blind*Point && upPrice != -1) || upPrice == -1)
      {
         if (upTicket != -1)
         {
            Print("d3 = ", upTicket);
            if (OrderDelete(upTicket)) upTicket = -1;
         }
         if (upTicket == -1 && upCount < MaxCount || MaxCount <= 0) OpenBuyStop(Magic, NormalizeDouble(up + 50*Point, Digits), TP, SL, Lots);
      }
   }
   
   if (isDn)
   {
      if ((MathAbs(dn - dnPrice) > Blind*Point && dnPrice != -1) || dnPrice == -1)
      {
         if (dnTicket != -1)
         {
            Print("d3 = ", dnTicket);
            if (OrderDelete(dnTicket)) dnTicket = -1;
         }
         if (dnTicket == -1 && dnCount < MaxCount || MaxCount <= 0) OpenSellStop(Magic, NormalizeDouble(dn - 50*Point, Digits), TP, SL, Lots);
      }
   }
}


void AddTarget(double lo, double hi)
{
   Print("Target found. Ask = ", Ask, ", Bid = ", Bid, ", lo = ", lo, ", hi = ", hi);
   int size = ArraySize(_Highs);
   
   ArrayResize(_Highs, size + 1);
   ArrayResize(_Lows, size + 1);
   
   _Highs[size] = hi;
   _Lows[size] = lo;
}

bool FindDown(double& value, double price)
{
   int size = ArraySize(_Highs);
   
   if (size == 0) return (false);
   
   double hi = -1;
   for (int i = 0; i < size; i++)
   {
      if (price > _Highs[i] && _Highs[i] > hi)
      {
         hi = _Highs[i];
      }
   }
   
   if (hi == -1) return (false);
   
   value = hi;
   return (true);
}

bool FindUp(double& value, double price)
{
   int size = ArraySize(_Lows);
   
   if (size == 0) return (false);
   
   double lo = 100000000;
   for (int i = 0; i < size; i++)
   {
      if (price < _Lows[i] && _Lows[i] < lo)
      {
         lo = _Lows[i];
      }
   }
   
   if (lo == -1) return (false);
   
   value = lo;
   return (true);
}

void DrawLevels(string id, double min, double max, double array[], color clr)
{
   if (IsTesting() && !IsVisualMode()) return;
   int size = ArraySize(array);
   for (int i = 0; i < size; i++)
   {
      double value = min + i*(max - min)/99.0;
      datetime time1 = Time[0] + 6000*Period();
      datetime time2 = Time[0] + 60*(100 - 10*array[i])*Period();
      
      ObjectCreate(id + i, OBJ_TREND, 0, time1, value, time2, value);
      ObjectSet(id + i, OBJPROP_TIME1, time1);
      ObjectSet(id + i, OBJPROP_TIME2, time2);
      ObjectSet(id + i, OBJPROP_PRICE1, value);
      ObjectSet(id + i, OBJPROP_PRICE2, value);
      ObjectSet(id + i, OBJPROP_COLOR, clr);
      ObjectSet(id + i, OBJPROP_RAY, false);
   }
}

void Smooth(double& a[])
{
   double res[100];
   res[0] = a[0];
   res[99] = a[99];
   
   for (int i = 1; i < 99; i++)
   {
      res[i] = a[i - 1]*0.2 + a[i]*0.6 + a[i + 1]*0.2;
   }
   for (i = 1; i < 99; i++)
   {
      a[i] = res[i];
   }
}